import re
import random
import nltk
from nltk.tokenize import word_tokenize
from cleantweet import CleanTweet
from collections import Counter, defaultdict
nltk.download('punkt', quiet=True)
nltk.download('stopwords', quiet=True)
nltk.download('punkt_tab', quiet=True)


class PredictionTweet(CleanTweet):
    def _ngram_model(self, n: int = 2):
        ngrams = [tuple(self.clean().split(' ')[i:i + n]) for i in range(len(self.clean().split(' ')) - n + 1)]
        model = defaultdict(Counter)
        # print(ngrams)
        for ngram in ngrams:
            prefix, next_word = ngram[:-1], ngram[-1]
            model[prefix][next_word] += 1
        return model

    def ngram(self, start_word: str, ngram_size: int = 2, max_length: int = 10):
        """
        a method to use the bag of words ngram model for next word prediction.
        :param start_word: the starting word for the prediction
        :param ngram_size: amount of ngrams you want, default is 2 (recommended)
        :param max_length: length of the predicted word or sentence
        :return: a predicted string of words
        """
        sentence = [start_word]
        prefix = tuple(sentence[-(ngram_size-1):])

        for _ in range(max_length):
            ngram_model = self._ngram_model(n=ngram_size)
            ngram_list = list(ngram_model.keys())
            ngram_list = [item for tup in ngram_list for item in tup]
            if prefix[0] not in ngram_list:
                print('Start Word is not in your Ngram model, choose a word that is presently in the model')
                return None

            next_word = random.choices(
                list(ngram_model[prefix].keys()),
                weights=ngram_model[prefix].values()
            )[0]
            sentence.append(next_word)

            if next_word in {'.', '!', '?'}:
                break

            prefix = tuple(sentence[-(ngram_size-1):])

        return ' '.join(sentence)

    def ngram_count(self):
        """
        method to know the ngram count
        :return: an integer of the ngram count
        """
        ngrams = self._ngram_model()
        ngram_count = list(ngrams.keys())
        ngram_count = [item for tup in ngram_count for item in tup]
        return len(ngram_count)

    def hmm(self):
        pass

    def rnn(self):
        pass

    def transformer(self):
        pass

    def bert_context(self):
        pass






import pandas as pd


class Datasets:
    def __init__(self):
        self.datasets = {
            "imdb": "https://ai.stanford.edu/~amaas/data/sentiment/aclImdb_v1.tar.gz",
            "twitter": "https://github.com/sbhatti/twitter-sentiment-analysis-dataset/raw/master/data.csv",
            "ag_news": "https://raw.githubusercontent.com/mhjabreel/CharCnn_Keras/master/data/ag_news_csv/train.csv",
            # Add more datasets as needed
        }

    def load_dataset(self, name):
        """
        Load a dataset by name.
        :param name: The name of the dataset to load.
        :return: A pandas DataFrame containing the dataset.
        """
        if name not in self.datasets:
            raise ValueError(f"Dataset '{name}' is not available.")

        url = self.datasets[name]
        return pd.read_csv(url)

    def list_datasets(self):
        """
        List all available datasets.
        :return: A list of dataset names.
        """
        return list(self.datasets.keys())



import random
import pandas as pd
from tqdm import tqdm
from cleantweet import CleanTweet
from cleantweet import StatsTweet


class SyntheticTweet:
    def __init__(self, text):
        self.text = text
        self.generated_sentences = []

    def rule_based(self, amount_of_sentences: int = 10, amount_of_pos: int = 5, percentage: float = 0.25):
        """
        this method uses a rule-based algorithm to generate synthetic data.
        :param amount_of_sentences: the amount of synthetic sentences you want to generate.
        :param amount_of_pos: the amount of each part of speech you want, you only need to choose
        one amount, it will be the same for nouns, verbs, and adverbs. The higher this amount,
        the more random your synthetic data will be.
        :param percentage: a float value that represents the salt value of the synthetic sentences that
        will be generated. It must be a float and the default is below 0.25 which will be ->
        noun -> verbs -> adverbs. Increasing the float value will give: noun -> verbs -> adverbs -> adjectives
        etc.
        :return: a list of the different synthetic data generated from the text object
        """

        print("Analyzing parts of speech...")
        figures_of_speech = StatsTweet(self.text)
        nouns = [figures_of_speech.amount_of_nouns(amount_of_pos)]
        pronouns = []
        verbs = [figures_of_speech.amount_of_verbs(amount_of_pos)]
        adjectives = [figures_of_speech.amount_of_adjectives(amount_of_pos)]
        adverbs = [figures_of_speech.amount_of_adverbs(amount_of_pos)]
        articles = ['the']
        subjects = [nouns + pronouns]
        predicates = [verbs]

        def sentence():
            if percentage > 0.25:
                synthetic_sentence = (f"{random.choice(nouns[0]['words'])} {random.choice(verbs[0]['words'])} "
                                      f"{random.choice(adverbs[0]['words'])} "
                                      f"{random.choice(nouns[0]['words'])} "
                                      f"{random.choice(adjectives[0]['words'])}")
            else:
                synthetic_sentence = (f"{random.choice(nouns[0]['words'])} "
                                      f"{random.choice(verbs[0]['words'])} "
                                      f"{random.choice(adverbs[0]['words'])} "
                                      f"{random.choice(nouns[0]['words'])}")

            return synthetic_sentence

        if nouns[0]['words'] == [] or verbs[0]['words'] == [] or adverbs[0]['words'] == []:
            print('Your text document must have a noun, a verb, and an adverb.')
            return []
        else:
            print(f"\nGenerating {amount_of_sentences} synthetic sentences...")
            self.generated_sentences = []
            for _ in tqdm(range(amount_of_sentences), desc="Generating sentences", unit="sentence"):
                self.generated_sentences.append(sentence())

            print("\nGeneration complete! Here are your synthetic sentences:")
            for sent in self.generated_sentences:
                print(sent)

            return self.generated_sentences

    def _amount_of_synthetic_data(self):
        """
        method for knowing the total count of synthetic data generated
        :return: The number of synthetic sentences generated in the last run
        """
        return len(self.generated_sentences)


import nltk
import pandas as pd
from cleantweet import CleanTweet
from nltk.tokenize import word_tokenize
from wordcloud import WordCloud
import matplotlib.pyplot as plt
nltk.download('punkt', quiet=True)
nltk.download('stopwords', quiet=True)
nltk.download('punkt_tab', quiet=True)


class DiagramTweet(CleanTweet):
    def word_cloud(self, width: int = 1400, height: int = 800, max_words: int = 100,
                   background_color: str = 'black', *args):
        """
        method to draw a wordcloud from a text/document.

        :param width: the width of the wordcloud, default is 1400px
        :param height: the height of the wordcloud, default is 800px
        :param max_words: how many words do you want to appear in the wordcloud, default is 100
        :param background_color: the background color of the wordcloud. You can enter the colors
        or hex codes as a string
        :param args: you can pass any other argument that is expected of the Wordcloud library method; as
        this method is a wrapper for that.
        :return: a wordcloud display image
        """

        words = self.clean()
        plt.imshow(WordCloud(width=width, height=height, max_words=max_words,
                             background_color=background_color).generate(words))
        plt.show()

    def frequency_distribution(self, amount: int = 2):
        """
        method for showing the frequency distribution (amount of times a particular word occurs) in the text/document
        :param amount: how many words do you want to know their frequency amount? the default is 2. The argument must
        be an integer and not a float value.
        :return: a frequency distribution image
        """

        tokens = word_tokenize(self.clean())
        text = nltk.FreqDist(tokens)
        text.plot(amount, cumulative=True)

    def tabulate(self, word_amount: int = 2):
        """
        method for showing the frequency distribution in a tabular format.
        :param word_amount: amount of words whose frequencies you want to see; default is 2
        :return: a horizontal table showing the related words and their corresponding frequencies
        """

        words = self.clean()
        words = nltk.word_tokenize(words)
        freq_dist = nltk.FreqDist(words)
        return freq_dist.tabulate(word_amount)

    def bar_chart(self, amount_of_bars: int = 10, bar_color: str = '#a4b2e2', background_color: str = 'lightgray',
                  xlabel_name: str = 'Word', ylabel_name: str = 'Frequency', title: str = 'Most Frequent Words',
                  x_ticks_rotation: int = 90, grid_color: str = '#272727', grid_alpha: float = 0.1):
        """
        a method for showing the bar chart with the word on the x-axis and their frequencies on the y-axis
        :param amount_of_bars: the amount of bars to show in the bar chart; default value is 10 and maximum is 20. It's
        advisable to keep it below 15 for pleasing visual results.
        :param bar_color: the color of the bars you want. It should be a string of either html color value names, or a
        valid color hex code.
        :param background_color: change the background color of the bar chart, the default value is lightgray.
        :param xlabel_name: the name of the xlabel, must be a string. The default value is Word.
        :param ylabel_name: the name of the ylabel, must be a string. The default value is Frequency
        :param title: the title of the bar graph, to add a title, pass a string as an argument.
        :param x_ticks_rotation: the angle you want the xticks to be in, default is 90 degrees.
        :param grid_color: the grid color you want, it must be a string.
        :param grid_alpha: how much transparency do you want, default is 0.1. it must be a floating point value.
        e.g. 0.1, 0.2 etc. if unsure, leave as it is.
        :return: a bar chart of the most frequent words in the document.

        """

        words = self.clean()
        words = nltk.word_tokenize(words)
        freq_dist = nltk.FreqDist(words)
        words_from_freq_dist = [word for word in freq_dist.keys()]
        values_from_freq_dist = [word for word in freq_dist.values()]

        data_freq = {xlabel_name: words_from_freq_dist, ylabel_name: values_from_freq_dist}
        dataframe = pd.DataFrame(data_freq)
        dataframe_sorted_freq = dataframe.sort_values(by=ylabel_name, ascending=False)

        # Plot the bar chart
        bars = plt.bar(dataframe_sorted_freq[xlabel_name][0:amount_of_bars],
                       dataframe_sorted_freq[ylabel_name][0:amount_of_bars],
                       color=bar_color, zorder=2)
        bars[(amount_of_bars//2)].set_color(background_color)
        plt.xlabel(xlabel_name)
        plt.ylabel(ylabel_name)
        plt.title(title)
        plt.xticks(rotation=x_ticks_rotation)
        plt.grid(color=grid_color, alpha=grid_alpha, zorder=1)
        plt.gca().spines['right'].set_visible(False)
        plt.gca().spines['top'].set_visible(False)
        plt.gca().spines['left'].set_visible(True)
        plt.gca().spines['bottom'].set_visible(True)
        plt.show()

    def pie_chart(self, amount_of_slices: int = 10, title: str = 'Word Frequency Distribution',
                  colors: list = None, autopct: str = '%1.1f%%', startangle: int = 90):
        """
        method for showing a pie chart of the most frequent words in the text
        :param amount_of_slices: the number of top words to display in the pie chart, default is 10
        :param title: the title of the pie chart, default is 'Word Frequency Distribution'
        :param colors: optional list of colors for the pie slices. If None, uses default matplotlib colors
        :param autopct: format string for percentage labels, default is '%1.1f%%'
        :param startangle: angle at which to start the first slice, default is 90 degrees
        :return: a pie chart visualization of word frequencies
        """
        words = self.clean()
        words = nltk.word_tokenize(words)
        freq_dist = nltk.FreqDist(words)
        most_common = freq_dist.most_common(amount_of_slices)
        
        labels = [word for word, freq in most_common]
        sizes = [freq for word, freq in most_common]
        
        plt.figure(figsize=(10, 8))
        plt.pie(sizes, labels=labels, autopct=autopct, startangle=startangle, colors=colors)
        plt.title(title)
        plt.axis('equal')
        plt.show()

    def lexicon_chart(self, amount_of_words: int = 20, title: str = 'Lexicon Distribution'):
        """
        method for showing a lexicon chart (word frequency visualization) of the text
        :param amount_of_words: the number of words to display, default is 20
        :param title: the title of the lexicon chart, default is 'Lexicon Distribution'
        :return: a lexicon chart visualization showing word frequencies
        """
        words = self.clean()
        words = nltk.word_tokenize(words)
        freq_dist = nltk.FreqDist(words)
        most_common = freq_dist.most_common(amount_of_words)
        
        words_list = [word for word, freq in most_common]
        frequencies = [freq for word, freq in most_common]
        
        plt.figure(figsize=(12, 6))
        plt.barh(words_list, frequencies)
        plt.xlabel('Frequency')
        plt.ylabel('Words')
        plt.title(title)
        plt.gca().invert_yaxis()
        plt.tight_layout()
        plt.show()

    def _bubble_chart(self):
        pass

    def _heatmap(self):
        pass




import re
import nltk
from cleantweet import CleanTweet
from nltk.tokenize import word_tokenize, sent_tokenize
from collections import Counter
nltk.download('punkt', quiet=True)
nltk.download('stopwords', quiet=True)
nltk.download('punkt_tab', quiet=True)
nltk.download('vader_lexicon', quiet=True)


class ClassificationTweet(CleanTweet):
    """
    Class for various text classification tasks including spam detection,
    hate speech detection, question answering, religious undertones, and racial tone classification
    """
    
    def __init__(self, text, *args):
        super().__init__(text, *args)
        self._spam_keywords = {
            'free', 'click', 'now', 'limited', 'offer', 'winner', 'congratulations',
            'prize', 'claim', 'urgent', 'act now', 'buy now', 'discount', 'save',
            'guaranteed', 'risk free', 'no credit check', 'make money', 'work from home'
        }
        
        self._hate_keywords = {
            'hate', 'kill', 'destroy', 'attack', 'violence', 'threat', 'harm',
            'stupid', 'idiot', 'moron', 'worthless', 'pathetic', 'disgusting'
        }
        
        self._religious_keywords = {
            'god', 'jesus', 'christ', 'allah', 'buddha', 'hindu', 'islam', 'christian',
            'bible', 'quran', 'prayer', 'pray', 'worship', 'faith', 'holy', 'sacred',
            'divine', 'blessed', 'amen', 'hallelujah', 'prophet', 'saint', 'angel',
            'heaven', 'hell', 'sin', 'salvation', 'redemption', 'spiritual'
        }
        
        self._racial_keywords = {
            'race', 'racial', 'ethnic', 'racist', 'discrimination', 'prejudice',
            'stereotype', 'minority', 'majority', 'white', 'black', 'asian', 'hispanic',
            'african', 'european', 'indigenous', 'native', 'immigrant', 'refugee'
        }

    def spam_detection(self, threshold: float = 0.3):
        """
        Method to detect spam content in the text
        :param threshold: threshold for spam detection (0.0 to 1.0), default is 0.3
        :return: a dictionary with spam score and classification
        """
        text_lower = self.clean().lower()
        words = word_tokenize(text_lower)
        word_set = set(words)
        
        # Count spam keywords
        spam_matches = len(word_set.intersection(self._spam_keywords))
        total_words = len(word_set)
        
        # Calculate spam score
        spam_score = spam_matches / max(total_words, 1)
        
        # Check for excessive capitalization (spam indicator)
        caps_ratio = sum(1 for c in self.clean() if c.isupper()) / max(len(self.clean()), 1)
        
        # Check for excessive punctuation
        punct_count = sum(1 for c in self.clean() if c in '!?')
        punct_ratio = punct_count / max(len(sent_tokenize(self.clean())), 1)
        
        # Combined spam score
        final_score = (spam_score * 0.5) + (min(caps_ratio, 0.3) * 0.3) + (min(punct_ratio, 0.2) * 0.2)
        
        is_spam = final_score >= threshold
        
        return {
            'is_spam': is_spam,
            'spam_score': round(final_score, 3),
            'keyword_matches': spam_matches,
            'total_words': total_words,
            'confidence': 'high' if abs(final_score - threshold) > 0.2 else 'medium'
        }

    def hate_speech_detection(self, threshold: float = 0.2):
        """
        Method to detect hateful or harmful speech in the text
        :param threshold: threshold for hate speech detection (0.0 to 1.0), default is 0.2
        :return: a dictionary with hate speech score and classification
        """
        text_lower = self.clean().lower()
        words = word_tokenize(text_lower)
        word_set = set(words)
        
        # Count hate keywords
        hate_matches = len(word_set.intersection(self._hate_keywords))
        total_words = len(word_set)
        
        # Calculate hate speech score
        hate_score = hate_matches / max(total_words, 1)
        
        # Check for aggressive punctuation
        aggressive_punct = sum(1 for c in self.clean() if c in '!?')
        punct_score = min(aggressive_punct / max(len(sent_tokenize(self.clean())), 1), 0.3)
        
        # Combined score
        final_score = (hate_score * 0.7) + (punct_score * 0.3)
        
        is_hateful = final_score >= threshold
        
        return {
            'is_hateful': is_hateful,
            'hate_score': round(final_score, 3),
            'keyword_matches': hate_matches,
            'total_words': total_words,
            'severity': 'high' if final_score > 0.4 else 'medium' if final_score > threshold else 'low'
        }

    def religious_undertones(self):
        """
        Method to detect religious undertones in the text
        :return: a dictionary with religious content analysis
        """
        text_lower = self.clean().lower()
        words = word_tokenize(text_lower)
        word_set = set(words)
        
        # Count religious keywords
        religious_matches = word_set.intersection(self._religious_keywords)
        match_count = len(religious_matches)
        total_words = len(word_set)
        
        # Calculate religious score
        religious_score = match_count / max(total_words, 1)
        
        # Identify which religions are mentioned
        religions_mentioned = []
        if any(word in text_lower for word in ['god', 'jesus', 'christ', 'bible', 'christian']):
            religions_mentioned.append('Christianity')
        if any(word in text_lower for word in ['allah', 'islam', 'quran', 'prophet', 'muslim']):
            religions_mentioned.append('Islam')
        if any(word in text_lower for word in ['buddha', 'buddhist', 'buddhism']):
            religions_mentioned.append('Buddhism')
        if any(word in text_lower for word in ['hindu', 'hinduism', 'veda']):
            religions_mentioned.append('Hinduism')
        
        return {
            'has_religious_content': match_count > 0,
            'religious_score': round(religious_score, 3),
            'keyword_matches': match_count,
            'matched_keywords': list(religious_matches),
            'religions_mentioned': religions_mentioned,
            'intensity': 'high' if religious_score > 0.1 else 'medium' if religious_score > 0.05 else 'low'
        }

    def racial_tone_classification(self):
        """
        Method to classify the racial tone of the text
        :return: a dictionary with racial tone analysis
        """
        text_lower = self.clean().lower()
        words = word_tokenize(text_lower)
        word_set = set(words)
        
        # Count racial keywords
        racial_matches = word_set.intersection(self._racial_keywords)
        match_count = len(racial_matches)
        total_words = len(word_set)
        
        # Calculate racial content score
        racial_score = match_count / max(total_words, 1)
        
        # Check for negative context
        negative_words = {'hate', 'against', 'oppose', 'reject', 'deny', 'blame'}
        negative_context = len(word_set.intersection(negative_words)) > 0
        
        # Determine tone
        if match_count == 0:
            tone = 'neutral'
        elif negative_context and racial_score > 0.05:
            tone = 'negative'
        elif racial_score > 0.05:
            tone = 'discussion'
        else:
            tone = 'neutral'
        
        return {
            'has_racial_content': match_count > 0,
            'racial_score': round(racial_score, 3),
            'keyword_matches': match_count,
            'matched_keywords': list(racial_matches),
            'tone': tone,
            'has_negative_context': negative_context
        }

    def question_answering(self, question: str):
        """
        Simple question answering method using keyword matching and sentence extraction
        :param question: the question to answer
        :return: a dictionary with the answer and relevant sentences
        """
        question_lower = question.lower()
        question_words = set(word_tokenize(question_lower))
        
        # Remove question words
        question_words = question_words - {'what', 'who', 'where', 'when', 'why', 'how', 'is', 'are', 'was', 'were', 'do', 'does', 'did', '?', 'the', 'a', 'an'}
        
        sentences = sent_tokenize(self.clean())
        sentence_scores = {}
        
        for sentence in sentences:
            sentence_lower = sentence.lower()
            sentence_words = set(word_tokenize(sentence_lower))
            
            # Score based on keyword overlap
            overlap = len(question_words.intersection(sentence_words))
            score = overlap / max(len(question_words), 1)
            sentence_scores[sentence] = score
        
        # Get top scoring sentences
        ranked_sentences = sorted(sentence_scores.items(), key=lambda x: x[1], reverse=True)
        top_sentences = [sent for sent, score in ranked_sentences[:3] if score > 0]
        
        # Generate answer
        if top_sentences:
            answer = top_sentences[0]
            confidence = 'high' if sentence_scores[answer] > 0.5 else 'medium' if sentence_scores[answer] > 0.2 else 'low'
        else:
            answer = "I couldn't find a relevant answer in the text."
            confidence = 'low'
        
        return {
            'answer': answer,
            'confidence': confidence,
            'relevant_sentences': top_sentences,
            'score': round(sentence_scores.get(answer, 0), 3) if top_sentences else 0
        }




from cleantweet.CleanTweet import CleanTweet
from cleantweet.DiagramTweet import DiagramTweet
from cleantweet.StatsTweet import StatsTweet
from cleantweet.SaveTweet import save_tweet
from cleantweet.SentimentTweet import SentimentTweet
from cleantweet.SyntheticTweet import SyntheticTweet
from cleantweet.PredictionTweet import PredictionTweet
from cleantweet.InferenceTweet import InferenceTweet
from cleantweet.ClassificationTweet import ClassificationTweet



import re
import nltk
from nltk.tokenize import word_tokenize
from nltk.sentiment import SentimentIntensityAnalyzer
from cleantweet import CleanTweet
nltk.download('punkt', quiet=True)
nltk.download('punkt_tab', quiet=True)
nltk.download('stopwords', quiet=True)
nltk.download('vader_lexicon', quiet=True)


class SentimentTweet(CleanTweet):
    """
    class for getting the sentiments contained within a body of text. Currently,
    the only implementation is the VADER algorithm
    """
    def vader(self):
        """
        Vader Sentiment method, this relies on the same algorithm used in nltk.
        :return: sentiment scores for each sentence in the body of text.
        """
        sentiment = SentimentIntensityAnalyzer()
        sentences = nltk.sent_tokenize(self.clean(tokenize_method=True))
        print(sentences)
        sentiment_scores = [sentiment.polarity_scores(sentence)['compound'] for sentence in sentences]
        return sentiment_scores

    def _bert(self):
        pass

    def _gloria(self):
        # this will be our custom sentiment model.
        pass



import re
import nltk
from collections import Counter
from cleantweet import CleanTweet
from nltk.tokenize import word_tokenize, sent_tokenize
nltk.download('punkt', quiet=True)
nltk.download('stopwords', quiet=True)
nltk.download('punkt_tab', quiet=True)


class InferenceTweet(CleanTweet):
    """
    Class for text summarization and inference tasks
    """
    
    def _fbs(self, amount_of_sentences: int = 5):
        """
        Frequency-Based Summarization (protected method)
        :param amount_of_sentences: number of sentences to extract
        :return: list of top sentences
        """
        sentences = sent_tokenize(self.clean())
        words = word_tokenize(self.clean().lower())
        word_frequency = Counter(words)
        sentence_scores = {sent: sum(word_frequency[word] for word in word_tokenize(sent.lower()))
                           for sent in sentences}
        ranked_sentences = sorted(sentence_scores, key=sentence_scores.get, reverse=True)
        return ranked_sentences[:amount_of_sentences]

    def frequency_based_summarization(self, amount_of_sentences: int = 5):
        """
        Frequency-Based Summarization - extracts most important sentences based on word frequency
        :param amount_of_sentences: number of sentences to extract for summary, default is 5
        :return: a list of summary sentences
        """
        summary_sentences = self._fbs(amount_of_sentences)
        return summary_sentences

    def textrank_summarization(self, amount_of_sentences: int = 5):
        """
        Graph-Based Summarization using TextRank algorithm
        :param amount_of_sentences: number of sentences to extract for summary, default is 5
        :return: a list of summary sentences
        """
        sentences = sent_tokenize(self.clean())
        if len(sentences) < amount_of_sentences:
            return sentences
        
        # Build similarity matrix
        similarity_matrix = self._build_similarity_matrix(sentences)
        
        # Calculate sentence scores using PageRank-like algorithm
        scores = self._calculate_textrank_scores(similarity_matrix)
        
        # Rank sentences by score
        ranked_sentences = sorted(zip(sentences, scores), key=lambda x: x[1], reverse=True)
        
        # Return top sentences in original order
        top_indices = sorted([sentences.index(sent) for sent, _ in ranked_sentences[:amount_of_sentences]])
        return [sentences[i] for i in top_indices]

    def _build_similarity_matrix(self, sentences):
        """
        Build similarity matrix between sentences
        :param sentences: list of sentences
        :return: similarity matrix
        """
        n = len(sentences)
        similarity_matrix = [[0.0] * n for _ in range(n)]
        
        for i in range(n):
            for j in range(n):
                if i != j:
                    similarity_matrix[i][j] = self._sentence_similarity(sentences[i], sentences[j])
        
        return similarity_matrix

    def _sentence_similarity(self, sent1, sent2):
        """
        Calculate similarity between two sentences using word overlap
        :param sent1: first sentence
        :param sent2: second sentence
        :return: similarity score
        """
        words1 = set(word_tokenize(sent1.lower()))
        words2 = set(word_tokenize(sent2.lower()))
        
        if len(words1) == 0 or len(words2) == 0:
            return 0.0
        
        intersection = words1.intersection(words2)
        union = words1.union(words2)
        
        return len(intersection) / len(union) if len(union) > 0 else 0.0

    def _calculate_textrank_scores(self, similarity_matrix, damping=0.85, max_iter=100, tol=1e-6):
        """
        Calculate TextRank scores using iterative algorithm
        :param similarity_matrix: similarity matrix between sentences
        :param damping: damping factor (default 0.85)
        :param max_iter: maximum iterations
        :param tol: tolerance for convergence
        :return: list of scores
        """
        n = len(similarity_matrix)
        scores = [1.0] * n
        
        # Normalize similarity matrix
        normalized_matrix = []
        for i in range(n):
            row_sum = sum(similarity_matrix[i])
            if row_sum > 0:
                normalized_matrix.append([sim / row_sum for sim in similarity_matrix[i]])
            else:
                normalized_matrix.append([0.0] * n)
        
        for _ in range(max_iter):
            prev_scores = scores[:]
            for i in range(n):
                score = 1 - damping
                for j in range(n):
                    if normalized_matrix[j][i] > 0:
                        score += damping * normalized_matrix[j][i] * prev_scores[j]
                scores[i] = score
            
            if sum(abs(scores[i] - prev_scores[i]) for i in range(n)) < tol:
                break
        
        return scores

    def summarize(self, method: str = 'frequency', amount_of_sentences: int = 5):
        """
        Main summarization method that supports different algorithms
        :param method: summarization method - 'frequency' or 'textrank', default is 'frequency'
        :param amount_of_sentences: number of sentences in summary, default is 5
        :return: summary as a string
        """
        if method.lower() == 'frequency':
            summary_sentences = self.frequency_based_summarization(amount_of_sentences)
        elif method.lower() == 'textrank':
            summary_sentences = self.textrank_summarization(amount_of_sentences)
        else:
            raise ValueError("Method must be 'frequency' or 'textrank'")
        
        return ' '.join(summary_sentences)




import re
import nltk
from nltk.tokenize import word_tokenize
nltk.download('punkt', quiet=True)
nltk.download('stopwords', quiet=True)
nltk.download('punkt_tab', quiet=True)


class CleanTweet:
    def __init__(self, text, *args):
        """
        a class for cleaning and preprocessing the text
        :param text: .txt file containing the textual data you want to use for the Natural Language Processing task.
        :param args: pass any argument that you want (leave this empty).
        """
        self.text = text
        self.word_count = 0
        self.special_characters = ""

    def clean(self, tokenize_method: bool = False):
        """
        method to clean the text
        :param tokenize_method: if set to True, it wil sentence tokenize else it will
        word tokenize. The default is word tokenize
        :return: the word tokens as a string
        """
        with open(self.text, "r", encoding='utf8', errors='ignore') as read_object:
            lines = read_object.read()

            # remove special characters and punctuations
            lines = re.sub('#', '', lines)
            lines = re.sub('\\n\\n', '', lines)
            lines = re.sub('[\\n\\n]', '', lines)
            lines = re.sub('(\n\n)', '', lines)
            lines = re.sub('[{}:_@\[\]0-9,%&*""?!/-]', '', lines)

            # remove the id and text tag
            lines = re.sub('(id)', '', lines)
            lines = re.sub('(text)', '', lines)
            lines = re.sub('(RT)', '', lines)

            # remove paragraph space/indentation
            lines = re.sub('  ', '', lines)
            if tokenize_method is True:
                lines = nltk.sent_tokenize(lines)
                self.word_count = len(lines)
                lines = ' '.join(lines)
            else:
                lines = word_tokenize(lines)
                lines = [line for line in lines if line.isalpha()]
                self.word_count = len(lines)
                lines = ' '.join(lines)
        return lines

    def show_word_collocations(self):
        """
        method to show the corresponding Word Collocations in the document
        :return: a list of the word collocations
        """
        text = nltk.Text(self.text)
        return text.collocation_list()

    def remove_curse_words(self):
        """
        method to remove curse words like 'fuck', 'hell', and 'damn'
        :return: a list of appropriate words in the text/corpus
        """
        data = self.clean()
        data_list = data.split()
        for datum in range(0, len(data_list)):
            if data_list[datum].strip().lower() == 'fuck' or data_list[datum].strip().lower() == 'hell'\
                    or data_list[datum].strip().lower() == 'damn':
                data_list[datum] = '****'
        return data_list

    def remove_stopwords(self):
        """
        Method to remove common English stopwords from the text
        :return: text with stopwords removed as a string
        """
        from nltk.corpus import stopwords
        nltk.download('stopwords', quiet=True)
        stop_words = set(stopwords.words('english'))

        words = self.clean().split()
        filtered_words = [
            word for word in words if word.lower() not in stop_words]
        return ' '.join(filtered_words)

    def remove_conjunctions(self):
        """
        Method to remove coordinating conjunctions (and, but, or, nor, for, yet, so) from the text
        :return: text with conjunctions removed as a string
        """
        conjunctions = {'and', 'but', 'or', 'nor', 'for', 'yet', 'so'}
        words = self.clean().split()
        filtered_words = [
            word for word in words if word.lower() not in conjunctions]
        return ' '.join(filtered_words)

    def remove_pronouns(self):
        """
        Method to remove common pronouns (I, me, my, mine, we, us, our, etc.) from the text
        :return: text with pronouns removed as a string
        """
        pronouns = {
            'i', 'me', 'my', 'mine', 'myself',
            'we', 'us', 'our', 'ours', 'ourselves',
            'you', 'your', 'yours', 'yourself', 'yourselves',
            'he', 'him', 'his', 'himself',
            'she', 'her', 'hers', 'herself',
            'it', 'its', 'itself',
            'they', 'them', 'their', 'theirs', 'themselves'
        }
        words = self.clean().split()
        filtered_words = [
            word for word in words if word.lower() not in pronouns]
        return ' '.join(filtered_words)

    def remove_custom_words(self, custom_words):
        """
        Method to remove any custom list of words provided by the user
        :param custom_words: set or list of words to remove
        :return: text with custom words removed as a string
        """
        if not isinstance(custom_words, (set, list)):
            raise ValueError("custom_words must be a set or list of strings")

        custom_words = set(word.lower() for word in custom_words)
        words = self.clean().split()
        filtered_words = [
            word for word in words if word.lower() not in custom_words]
        return ' '.join(filtered_words)

    def remove_articles(self):
        """
        Method to remove articles (a, an, the) from the text
        :return: text with articles removed as a string
        """
        articles = {'a', 'an', 'the'}
        words = self.clean().split()
        filtered_words = [
            word for word in words if word.lower() not in articles]
        return ' '.join(filtered_words)

    def remove_prepositions(self):
        """
        Method to remove common prepositions from the text
        :return: text with prepositions removed as a string
        """
        prepositions = {
            'in', 'on', 'at', 'to', 'for', 'with', 'by', 'of',
            'from', 'about', 'against', 'between', 'into', 'through',
            'during', 'before', 'after', 'above', 'below', 'up',
            'down', 'under', 'over', 'again', 'further', 'beyond'
        }
        words = self.clean().split()
        filtered_words = [
            word for word in words if word.lower() not in prepositions]
        return ' '.join(filtered_words)
